#! python3
# -*- encoding: utf-8 -*-
'''
Current module: webuidriver.cli

Rough version history:
v1.0    Original version to use

********************************************************************
    @AUTHOR:  Administrator-Bruce Luo(罗科峰)
    MAIL:    lkf20031988@163.com
    RCS:      webuidriver.cli,v 1.0 2018年8月19日
    FROM:   2018年8月19日
********************************************************************

======================================================================

UI and Web Http automation frame for python.

'''


import argparse
from rtsf.p_applog import color_print,logger
from rtsf.p_executer import TestRunner
from webuidriver.driver import LocalDriver, RemoteDriver
from webuidriver.remote.SeleniumHatch import SeleniumHatch
from webuidriver.remote.SeleniumJar import SeleniumJar
from webuidriver.__about__ import __version__
    

def hub_main_run():
    parser = argparse.ArgumentParser(description="selenium server jar with hub mode.")
                
    parser.add_argument(
        '--java-path', default='java',
        help="path of java. default is `java` if JAVA_HOME is configured. ")
        
    parser.add_argument(
        '--port', type = int, default = 4444,
        help="listen port for hub mode. default port: 4444")
                
    parser.add_argument(
        'server_jar', 
        help="selenium server jar path for selenium grid mode")
    
    
    color_print("webuidriver {}".format(__version__), "GREEN")
    args = parser.parse_args()
    SeleniumJar(args.server_jar, args.java_path).hub(args.port).start_server()
    
def node_main_run():
    parser = argparse.ArgumentParser(description="selenium server jar with node mode.")
                
    parser.add_argument(
        '--java-path', default='java',
        help="path of java. default is `java` if JAVA_HOME is configured. ")
        
    parser.add_argument(
        '--port', type = int, default = 5555,
        help="listen port for node mode. default port: 5555")
    
    parser.add_argument(
        '--hub-ip', default = "localhost",
        help="hub host or hub ip which need to connect. default host: localhost")
    
    parser.add_argument(
        '--hub-port', type = int, default = 4444,
        help="hub port which need to connect. default: 4444")
    
                
    parser.add_argument(
        'server_jar', 
        help="selenium server jar path for selenium grid mode")
    
    
    color_print("webuidriver {}".format(__version__), "GREEN")
    args = parser.parse_args()
    SeleniumJar(args.server_jar, args.java_path).node(args.port,(args.hub_ip, args.hub_port)).start_server()
    
    

def local_main_run():
    
    parser = argparse.ArgumentParser(description="Tools for web ui test. Base on rtsf.")
            
    parser.add_argument(
        '--log-level', default='INFO',
        help="Specify logging level, default is INFO.")
    
    parser.add_argument(
        '--log-file',
        help="Write logs to specified file path.")
    
    parser.add_argument(
        '--browser',default = "chrome",
        help="set browser, only chrome or firefox. default: chrome")
    
    parser.add_argument(
        '--download-path',default = None,
        help="set the path where to save download file from browser. default: None")
    
    parser.add_argument(
        '--marionette', action = 'store_true', default = False,
        help="use firefox driver `geckodriver.exe` if True. default: False")
    
    parser.add_argument(
        'case_file', 
        help="yaml testcase file")
    
    color_print("webuidriver {}".format(__version__), "GREEN")
    args = parser.parse_args()
    logger.setup_logger(args.log_level, args.log_file)    
    
    LocalDriver._browser = args.browser
    LocalDriver._download_path = args.download_path
    LocalDriver._marionette = args.marionette
    runner = TestRunner(runner = LocalDriver).run(args.case_file)
    html_report = runner.gen_html_report()
    color_print("report: {}".format(html_report))
    
def remote_main_run():
    
    parser = argparse.ArgumentParser(description="Tools for web ui test. Base on rtsf.")
            
    parser.add_argument(
        '--log-level', default='INFO',
        help="Specify logging level, default is INFO.")
    
    parser.add_argument(
        '--log-file',
        help="Write logs to specified file path.")
    
    parser.add_argument(
        '--browser',default = "chrome",
        help="set browser, only chrome or firefox. default: chrome")
    
    parser.add_argument(
        '--download-path',default = None,
        help="set the path where to save download file from browser. default: None")
    
    parser.add_argument(
        '--marionette', action = 'store_true', default = False,
        help="use firefox driver `geckodriver.exe` if True. default: False")
    
    parser.add_argument(
        '--ip',default = "localhost",
        help="remote hub ip. default: localhost")
    
    parser.add_argument(
        '--port', type = int, default = 4444,
        help="remote hub port. default: 4444")
    
    parser.add_argument(
        'case_file', 
        help="yaml testcase file")
    
    color_print("webuidriver {}".format(__version__), "GREEN")
    args = parser.parse_args()
    logger.setup_logger(args.log_level, args.log_file)    
    
    RemoteDriver._browser = args.browser
    RemoteDriver._download_path = args.download_path
    RemoteDriver._marionette = args.marionette
    RemoteDriver._remote_ip = args.ip
    RemoteDriver._remote_port = args.port
    
    runner = TestRunner(runner = RemoteDriver).run(args.case_file)
    html_report = runner.gen_html_report()
    color_print("report: {}".format(html_report))
    
    
    
    



# -*- encoding: utf-8 -*-
'''
Current module: rock4.softtest.web.SeleniumHatch

Rough version history:
v1.0    Original version to use

********************************************************************
    @AUTHOR:  Administrator-Bruce Luo(罗科峰)
    MAIL:    lkf20031988@163.com
    RCS:      rock4.softtest.web.SeleniumHatch,v 1.0 2017年2月27日
    FROM:   2017年2月27日
********************************************************************

======================================================================

UI and Web Http automation frame for python.

'''

import requests,re
from selenium import webdriver
from selenium.webdriver import DesiredCapabilities

        
class SeleniumHatch(object):
    ''' Hatch remote browser driver from selenium grid '''
        
    @staticmethod
    def get_remote_executors(hub_ip, port = 4444):
        ''' Get remote hosts from Selenium Grid Hub Console
        @param hub_ip: hub ip of selenium grid hub
        @param port: hub port of selenium grid hub
        '''
        resp = requests.get("http://%s:%s/grid/console" %(hub_ip, port))
        
        remote_hosts = ()
        if resp.status_code == 200:
            remote_hosts = re.findall("remoteHost: ([\w/\.:]+)",resp.text)
        return [host + "/wd/hub" for host in remote_hosts]
    
    @staticmethod
    def get_remote_browser_capabilities(browser = "chrome", download_path = None, marionette = False):
        '''        
        @param browser: firefox chrome opera safari internetexplorer edge htmlunit htmlunitwithjs
        @param marionette: use firefox's geckodriver if True. selenium 3.x开始，webdriver/firefox/webdriver.py的__init__中，executable_path="geckodriver"; 而2.x是executable_path="wires"; 也就是说， firefox 47以上版本，需要下载第三方driver，即geckodriver。
        @param download_path:  set a download path for browser.
        @return:  return the capabilities of this browser 
        '''
        
        browser = browser.upper()            
        cap = getattr(DesiredCapabilities, browser).copy() 
                
        if browser == "FIREFOX":
            cap['marionette'] = marionette;#          
            if download_path:
                fp = webdriver.FirefoxProfile() 
                fp.set_preference("browser.download.folderList", 2);# 设置Firefox的默认 下载 文件夹。0是桌面；1是“我的下载”；2是自定义
                fp.set_preference("browser.download.manager.showWhenStarting", False)
                fp.set_preference("browser.download.dir", download_path)
                fp.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
                cap['firefox_profile'] = fp
                                      
        elif browser == "CHROME":
            options = webdriver.ChromeOptions()
            options.add_experimental_option("excludeSwitches", ["ignore-certificate-errors"])
            if download_path:
                prefs = {"download.default_directory": download_path}
                options.add_experimental_option("prefs", prefs)
            cap = options.to_capabilities()
        return cap
    
    @staticmethod
    def gen_remote_driver(executor, capabilities):
        ''' Generate remote drivers with desired capabilities(self.__caps) and command_executor
        @param executor: command executor for selenium remote driver
        @param capabilities: A dictionary of capabilities to request when starting the browser session.
        @return: remote driver
        '''        
        # selenium requires browser's driver and PATH env. Firefox's driver is required for selenium3.0            
        firefox_profile = capabilities.pop("firefox_profile",None)            
        return webdriver.Remote(executor, desired_capabilities=capabilities, browser_profile = firefox_profile)
    
    @staticmethod
    def gen_local_driver(browser, capabilities):
        ''' Generate localhost drivers with desired capabilities(self.__caps)
        @param browser:  firefox or chrome
        @param capabilities:  A dictionary of capabilities to request when starting the browser session.
        @return:  localhost driver
        '''
        if browser == "firefox":
            fp = capabilities.pop("firefox_profile",None)
            return webdriver.Firefox(desired_capabilities =capabilities, firefox_profile=fp)
                   
        elif browser == "chrome":            
            return webdriver.Chrome(desired_capabilities=capabilities)
        
        else:
            raise TypeError("Unsupport browser {}".format(browser))
            
    
    



#! python3
# -*- encoding: utf-8 -*-

import re

from rtsf.p_executer import Runner
from rtsf.p_common import CommonUtils, ModuleUtils, FileSystemUtils
from rtsf.p_exception import FunctionNotFound, VariableNotFound
from webuidriver.remote.SeleniumHatch import SeleniumHatch

        
class _Driver(Runner):     
    def __init__(self, is_local_driver):
        super(_Driver, self).__init__()
        self._local_driver = is_local_driver
    
    def run_test(self, testcase_dict, variables, driver_map):
        fn, fn_driver = driver_map        
        parser = self.parser        
        tracer = self.tracers[fn]
        
        _Actions = ModuleUtils.get_imported_module("webuidriver.actions")
        _Actions.Web.driver = fn_driver
        
        functions = {}
        web_functions = ModuleUtils.get_callable_class_method_names(_Actions.Web)
        web_element_functions = ModuleUtils.get_callable_class_method_names(_Actions.WebElement)
        web_context_functions = ModuleUtils.get_callable_class_method_names(_Actions.WebContext)
        web_wait_functions = ModuleUtils.get_callable_class_method_names(_Actions.WebWait)
        web_verify_functions = ModuleUtils.get_callable_class_method_names(_Actions.WebVerify)
        web_actions_functions = ModuleUtils.get_callable_class_method_names(_Actions.WebActions)
        functions.update(web_functions)
        functions.update(web_element_functions)
        functions.update(web_context_functions)
        functions.update(web_wait_functions)
        functions.update(web_verify_functions)
        functions.update(web_actions_functions)   
        parser.bind_functions(functions)
        
        _Actions.WebContext.glob.update(variables)
        parser.update_binded_variables(_Actions.WebContext.glob)        
        
        case_name = FileSystemUtils.get_legal_filename(parser.eval_content_with_bind_actions(testcase_dict["name"]))
        tracer.start(self.proj_info["module"], case_name,
                     testcase_dict.get("responsible", "Administrator"),
                     testcase_dict.get("tester", "Administrator")
                     )
        tracer.section(case_name)
        
        try:
            tracer.normal("**** bind glob variables")                
            glob_vars = parser.eval_content_with_bind_actions(testcase_dict.get("glob_var", {}))
            tracer.step("set global variables: {}".format(glob_vars))                
            _Actions.WebContext.glob.update(glob_vars)            
             
            tracer.normal("**** bind glob regular expression")
            globregx = {k: re.compile(v) for k, v in testcase_dict.get("glob_regx", {}).items()}
            tracer.step("set global regular: {}".format(globregx))            
            _Actions.WebContext.glob.update(globregx)
                             
            tracer.normal("**** precommand")
            precommand = testcase_dict.get("pre_command", [])
            parser.eval_content_with_bind_actions(precommand)
            for i in precommand:
                tracer.step("{}".format(i))
             
            tracer.normal("**** steps")
            steps = testcase_dict["steps"]
            for step in steps:
                # print("---")
                if "webdriver" not in step:
                    continue
                
                if not step["webdriver"].get("action"):
                    raise KeyError("webdriver.action")            
                
                # print(step)
                if step["webdriver"].get("by"):
                    by = parser.eval_content_with_bind_actions(step["webdriver"].get("by"))
                    tracer.normal("preparing: by -> {}".format(by))
                    
                    value = parser.eval_content_with_bind_actions(step["webdriver"].get("value"))
                    tracer.normal("preparing: value -> {}".format(value))
                    
                    index = parser.eval_content_with_bind_actions(step["webdriver"].get("index", 0))
                    tracer.normal("preparing: index -> {}".format(index))
                    
                    timeout = parser.eval_content_with_bind_actions(step["webdriver"].get("timeout", 10))
                    tracer.normal("preparing: timeout -> {}".format(timeout))                           
                
                    prepare = parser.get_bind_function("SetControl")
                    prepare(by=by, value=value, index=index, timeout=timeout)
                                
                result = parser.eval_content_with_bind_actions(step["webdriver"]["action"])
                # print(":",result)
                if result is False:
                    tracer.fail(step["webdriver"]["action"])
                else:
                    tracer.ok(step["webdriver"]["action"])
                        
            tracer.normal("**** postcommand")
            postcommand = testcase_dict.get("post_command", [])        
            parser.eval_content_with_bind_actions(postcommand)
            for i in postcommand:
                tracer.step("{}".format(i))
            
            tracer.normal("**** verify")
            verify = testcase_dict.get("verify", [])
            result = parser.eval_content_with_bind_actions(verify)
            for v, r in zip(verify, result):
                if r is False:
                    tracer.fail(u"{} --> {}".format(v, r))
                else:
                    tracer.ok(u"{} --> {}".format(v, r))
                        
        except KeyError as e:
            tracer.error("Can't find key[%s] in your testcase." % e)
        except FunctionNotFound as e:
            tracer.error(e)
        except VariableNotFound as e:
            tracer.error(e)
        except Exception as e:
            tracer.error("%s\t%s" % (e, CommonUtils.get_exception_error()))
        finally:
            # tracer.normal("globals:\n\t{}".format(parser._variables))
            tracer.stop()
        return tracer

    @property
    def drivers(self):
        """return [(device_id: driver)]"""
        return self._default_drivers


class LocalDriver(_Driver):
    _browser = "chrome"
    _download_path = None
    _marionette = False
    
    def __init__(self):
        super(LocalDriver, self).__init__(is_local_driver=True)
        
        chrome_capabilities = SeleniumHatch.get_remote_browser_capabilities(browser=LocalDriver._browser,
                                                                            download_path=LocalDriver._download_path, 
                                                                            marionette=LocalDriver._marionette)
        self._default_drivers = [
            ("", SeleniumHatch.gen_local_driver(browser="chrome", capabilities=chrome_capabilities))
        ]


class RemoteDriver(_Driver):
    _browser = "chrome"
    _download_path = None
    _marionette = False
    _remote_ip = "localhost"
    _remote_port = 4444
    
    def __init__(self):
        super(RemoteDriver, self).__init__(is_local_driver=False)
                
        self._default_devices = []
        
        self._default_drivers = []        
        executors = SeleniumHatch.get_remote_executors(RemoteDriver._remote_ip, RemoteDriver._remote_port)
        chrome_capabilities = SeleniumHatch.get_remote_browser_capabilities(browser=RemoteDriver._browser,
                                                                            download_path=RemoteDriver._download_path,
                                                                            marionette=RemoteDriver._marionette)
        for executor in executors:
            fn = FileSystemUtils.get_legal_filename(executor)
            self._default_devices.append(fn)            
            self._default_drivers.append((fn, SeleniumHatch.gen_remote_driver(executor, chrome_capabilities)))            



# -*- encoding: utf-8 -*-
'''
Current module: rock4.softtest.web.actions

Rough version history:
v1.0    Original version to use
v2.0    use rtsf for web driver

********************************************************************
    @AUTHOR:  Administrator-Bruce Luo(罗科峰)
    MAIL:    lkf20031988@163.com
    RCS:      rock4.softtest.web.actions,v 2.0 2018年8月21日
    FROM:   2017年2月18日
********************************************************************

======================================================================

UI and Web Http automation frame for python.

'''

from selenium.webdriver import ActionChains
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support.select import Select
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By

import time,os,re,json,ast

def _parse_string_value(str_value):
    try:
        return ast.literal_eval(str_value)
    except ValueError:
        return str_value
    except SyntaxError:
        return str_value
           
    
class Web(object):
    driver = None    
    
    @staticmethod
    def NavigateTo(url):
        Web.driver.get(url)
        
    @staticmethod
    def PageSource():
        ''' page source for this web '''
        return Web.driver.page_source
        
    @staticmethod
    def Refresh():
        Web.driver.refresh()
    
    @staticmethod
    def Forward():
        Web.driver.forward()        
    
    @staticmethod
    def Back():
        Web.driver.back()
            
    @staticmethod
    def IESkipCertError():
        ''' IE Skip SSL Cert Error '''
        Web.driver.get("javascript:document.getElementById('overridelink').click();")
    
    @staticmethod
    def Maximize():
        """ 浏览器最大化       """
        Web.driver.maximize_window()
    
    @staticmethod
    def SetWindowSize(width, height):
        """ 设定浏览器宽高   """
        Web.driver.set_window_size(width, height)
                
    @staticmethod
    def SwitchToDefaultFrame():        
        try:
            Web.driver.switch_to.default_content()
        except:
            return False
    
    @staticmethod
    def SwitchToNewFrame(frame_name):
        try:            
            WebDriverWait(Web.driver, 10).until(lambda driver: getattr(driver,"switch_to.frame")(frame_name))          
        except:            
            print("Waring: Timeout at %d seconds.Frame %s was not found." %frame_name)
            return False       
    
    @staticmethod
    def NewTab(url=""):
        Web.Js("window.open('%s')" %url)
        Web.SwitchToNewWindow()        
    
    @staticmethod
    def SwitchToDefaultWindow():             
        try:
            default_win = Web.driver.window_handles[0]
            Web.driver.switch_to.window(default_win)
        except:
            return False
            
    @staticmethod
    def SwitchToNewWindow(): 
        try:
            WebDriverWait(Web.driver, 10).until(lambda driver: len(driver.window_handles) >= 2)            
            new_win = Web.driver.window_handles[-1]
            Web.driver.switch_to.window(new_win)
        except:            
            print("Waring: Timeout at %d seconds. Pop Window Not Found.")
            return False
    
    @staticmethod
    def SwitchToDefaultContext():
        ''' APP应用，切换到默认上下文 '''
        try:
            Web.driver.switch_to.context(Web.driver.contexts[0])
        except:
            return False
        
    @staticmethod
    def SwitchToNewContext():
        ''' APP应用，切换到新的上下文 '''
        try:
            WebDriverWait(Web.driver, 10).until(lambda driver: len(driver.contexts) >= 2)
            new_context = Web.driver.contexts[-1]
            Web.driver.switch_to.context(new_context)       
        except:            
            print("Waring: Timeout at 10 seconds. New context Not Found.")
            return False
        
    @staticmethod
    def SwitchToAlert(): 
        ''' <input value="Test" type="button" onClick="alert('OK')" > '''
        try:            
            alert = WebDriverWait(Web.driver, 10).until(lambda driver: driver.switch_to_alert())
            return alert            
        except:            
            print("Waring: Timeout at %d seconds.Alert was not found.")
            return False
            
    @staticmethod
    def AlertAccept():                
        alert = Web.SwitchToAlert()
        if alert:
            alert.accept()        
        Web.SwitchToDefaultFrame()
               
    @staticmethod
    def AlertDismiss():
        alert = Web.SwitchToAlert()
        if alert:
            alert.dismiss()
        Web.SwitchToDefaultFrame()
                        
    @staticmethod
    def AlertSendKeys(value):
        alert = Web.SwitchToAlert()
        if alert:
            alert.send_keys(value)
        Web.SwitchToDefaultFrame()
    
    @staticmethod
    def Js(script):
        """ 执行  js script       """
        getattr(Web.driver,"execute_script")(script)
        
    @staticmethod
    def ScrollTo(x, y):
        #p_log.step_info("normal",u"Element [%s]: Scroll To [%s, %s]" % (cls.__name__, x, y))
        # X-Y-top: window.scrollTo("0","0")
        # X-bottom:  window.scrollTo("10000","0"),   Y-bottom:  window.scrollTo("0","10000")           
        Web.driver.execute_script("window.scrollTo(%s, %s);" % (x, y))
            
    @staticmethod
    def ScreenShoot(f_path):
        return Web.driver.save_screenshot(f_path)
    
    @staticmethod
    def WebClose():
        try:
            Web.driver.close()
            Web.SwitchToDefaultWindow()       
        except:
            pass
            
    @staticmethod
    def WebQuit():
        try:
            Web.driver.quit()            
        except:
            pass
        finally:
            Web.driver=None    
                    
class WebElement(object):
    
    _control = {
        "by":None,
        "value":None, 
        "index":0,
        "timeout":10,
        } 
                
    @classmethod
    def SetControl(cls,**kwargs):
        cls._control.update(kwargs)
    
    @classmethod
    def GetControl(cls):
        return cls._control
                
    @classmethod
    def _element(cls):
        '''   find the element with controls '''
        if not cls._is_selector():
            raise Exception("Invalid selector[%s]." %cls._control["by"])
        
        driver = Web.driver
        try:            
            elements = WebDriverWait(driver, cls._control["timeout"]).until(lambda driver: getattr(driver,"find_elements")(cls._control["by"], cls._control["value"]))
        except:                        
            raise Exception("Timeout at %d seconds.Element(%s) not found." %(cls._control["timeout"],cls._control["by"]))
        
        if len(elements) < cls._control["index"] + 1:                    
            raise Exception("Element [%s]: Element Index Issue! There are [%s] Elements! Index=[%s]" % (cls.__name__, len(elements), cls._control["index"]))
        
        if len(elements) > 1:              
            print("Element [%s]: There are [%d] elements, choosed index=%d" %(cls.__name__,len(elements),cls._control["index"]))
        
        elm = elements[cls._control["index"]]
        cls._control["index"] = 0        
        return elm
    
    @classmethod
    def _elements(cls):
        '''   find the elements with controls '''
        if not cls._is_selector():
            raise Exception("Invalid selector[%s]." %cls._control["by"])
        
        driver = Web.driver
        try:            
            elements = WebDriverWait(driver, cls._control["timeout"]).until(lambda driver: getattr(driver,"find_elements")(cls._control["by"], cls._control["value"]))
        except:            
            raise Exception("Timeout at %d seconds.Element(%s) not found." %(cls._control["timeout"],cls._control["by"]))
            
        return elements
       
    
    @classmethod
    def _is_selector(cls):
        all_selectors = (By.CLASS_NAME, By.CSS_SELECTOR, By.ID, By.LINK_TEXT, By.NAME, By.PARTIAL_LINK_TEXT, By.TAG_NAME, By.XPATH)
                        
        if cls._control["by"] in all_selectors:
            return True
        
        print("Warning: selector[%s] should be in %s" %(cls._control["by"],all_selectors))
        return False

class WebContext(WebElement):
    
    glob = {}
            
    @classmethod
    def SetVar(cls, name, value):
        ''' set static value
        :param name: glob parameter name
        :param value: parameter value
        '''
        cls.glob.update({name:value})
                
    @classmethod
    def GetVar(cls, name):
        return cls.glob.get(name)     
    
    @classmethod
    def DyStrData(cls, name, regx, index = 0):
        ''' set dynamic value from the string data of response  
        @param name: glob parameter name
        @param regx: re._pattern_type
            e.g.
            DyStrData("a",re.compile('123'))
        '''
        text = Web.PageSource()
        if not text:
            return
        if not isinstance(regx, re._pattern_type):
            raise Exception("DyStrData need the arg which have compiled the regular expression.")
            
        values = regx.findall(text)
        result = ""
        if len(values)>index:
            result = values[index]        
        cls.glob.update({name:result})
                            
    @classmethod
    def DyJsonData(cls,name, sequence):
        ''' set dynamic value from the json data of response
        @note: 获取innerHTML json的数据 如，   <html><body>{  "code": 1,"desc": "成功"}</body></html>
        @param name: glob parameter name
        @param sequence: sequence for the json
            e.g.
            result={"a":1,
               "b":[1,2,3,4],
               "c":{"d":5,"e":6},
               "f":{"g":[7,8,9]},
               "h":[{"i":10,"j":11},{"k":12}]
               }
            
            sequence1 ="a" # -> 1
            sequence2 ="b.3" # -> 4
            sequence3 = "f.g.2" # -> 9
            sequence4 = "h.0.j" # -> 11
        '''
        
        cls.SetControl(by = "tag name", value = "body")        
        json_body  = cls._element().get_attribute('innerHTML')
                
        if not json_body:
            return
                
        resp = json.loads(json_body)                    
        sequence = [_parse_string_value(i) for i in sequence.split('.')]    
        for i in sequence:
            try:
                if isinstance(i, int):
                    resp = resp[i]   
                else:
                    resp = resp.get(i)
            except:            
                cls.glob.update({name:None})
                return        
        cls.glob.update({name:resp})
        
    @classmethod
    def DyAttrData(cls,name, attr):
        attr_value = cls._element().get_attribute(attr)
        cls.glob.update({name:attr_value})
                
    @classmethod
    def GetText(cls):        
        return cls._element().text
    
    @classmethod
    def GetAttribute(cls, attr):
        return cls._element().get_attribute(attr)

class WebWait(WebElement):    
    
    @classmethod
    def TimeSleep(cls, seconds):
        time.sleep(seconds)
        
    @classmethod
    def WaitForAppearing(cls):        
        try:
            result = True if cls._element() else False                            
        except:
            result = False
        return result
        
    @classmethod
    def WaitForDisappearing(cls):        
        try:
            result = False if cls._element() else True
        except:
            result = True
        return result
        
    @classmethod
    def WaitForVisible(cls):
        try:
            result = cls._element().is_displayed()
        except:
            result = False        
        return result
    
class WebVerify(WebElement):
    
    @classmethod
    def VerifyVar(cls, name, expect_value):
        if WebContext.GetVar(name) == expect_value:
            return True
        else:
            return False
        
    @classmethod
    def VerifyText(cls, text):
        # 元素text值，为text
        try:
            result = cls._element().text == text
        except:
            result = False
        return result
        
    @classmethod
    def VerifyTitle(cls, title):
        # 当前页面的title
        if Web.driver.title == title:
            return True
        else:
            print("VerifyTitle: %s" % Web.driver.title)
            return False
    
    @classmethod
    def VerifyURL(cls, url):
        """ 获取当前页面的url """
          
        if Web.driver.current_url == url:
            return True
        else:
            print("VerifyURL: %s" % Web.driver.current_url)
            return False
    
    @classmethod
    def VerifyAlertText(cls, txt_value):        
        alert = Web.SwitchToAlert()
        result = False    
        if alert and txt_value in alert.text:                        
            result = True
        Web.SwitchToDefaultFrame()
        
        if result == False:
            print("VerifyAlertText: %s" % alert.text)
            
        return result
        
    @classmethod
    def VerifyElemEnabled(cls):
        try:
            result = cls._element().is_enabled()                          
        except:
            result = False
        return result   
    
    @classmethod
    def VerifyElemNotEnabled(cls):
        try:
            result = False if cls._element().is_enabled() else True                          
        except:
            result = True
        return result   
    
    @classmethod
    def VerifyElemVisible(cls):
        #p_log.step_info("normal","Element [%s]: IsVisible?" % (cls.__name__))
        try:
            result = cls._element().is_displayed()
        except:
            result = False
        return result
    
    @classmethod
    def VerifyElemNotVisible(cls):
        #p_log.step_info("normal","Element [%s]: IsVisible?" % (cls.__name__))
        try:
            result = False if cls._element().is_displayed() else True
        except:
            result = True
        return result       
    
    @classmethod
    def VerifyElemInnerHtml(cls, expect_text):
        '''
        @note: verify content of inner html is expected.  
        @param expect_text: expect inner html value
        '''
        if expect_text == "": return
                
        inner_html = cls._element().get_attribute('innerHTML')
        if expect_text in inner_html:
            return True
        else:
            print("VerifyElemInnerHtml: %s" % inner_html)
            return False    
    
    @classmethod
    def VerifyElemAttr(cls, attr_name, expect_value):
        '''
        @note:  verify content of attribute is expected content
        @param attr_name: name of element attribute 
        @param expet_value: expect attribute value
        '''
        try:
            result = expect_value in cls._element().get_attribute(attr_name)
        except:
            result = False
        return result
    
    @classmethod
    def VerifyElemCounts(cls, num):        
        if len(cls._elements()) == num:
            return True
        else:
            return False
        

class WebActions(WebElement):
    ''' Web Element.(selenium)'''
            
    @classmethod
    def SendKeys(cls, value):
        '''
        @param value: 文本框，输入的文本
        '''
        if value == "":
            return
        
        element = cls._element()
        element.clear()        
        action = ActionChains(Web.driver)
        action.send_keys_to_element(element, value)
        action.perform()
        
        
    @classmethod    
    def SelectByText(cls, text):
        ''' 通过文本值，选择下拉框选项，
        @param text: 下拉框  文本
        '''
        try:
            Select(cls._element()).select_by_visible_text(text)
        except:
            return False
        
    @classmethod    
    def DeSelectByText(cls, text):
        ''' 通过文本值，取消选择下拉框选项，
        @param text: 下拉框  文本
        '''
        try:
            Select(cls._element()).deselect_by_visible_text(text)
        except:
            return False
    
    
    @classmethod    
    def SelectByIndex(cls, index):
        ''' 通过索引，选择下拉框选项，
        @param index: 下拉框  索引
        '''
        try:
            Select(cls._element()).select_by_index(int(index))
        except:
            return False
                
    @classmethod    
    def DeSelectByIndex(cls, index):
        ''' 通过索引，取消选择下拉框选项，
        @param index: 下拉框  索引
        '''
        try:
            Select(cls._element()).deselect_by_index(int(index))
        except:
            return False
            
    @classmethod    
    def SelectByValue(cls, value):
        ''' 通过value值，选择下拉框选项，
        @param value: 下拉框  value属性 值
        '''
        try:
            Select(cls._element()).select_by_value(value)
        except:
            return False
                
    @classmethod    
    def DeSelectByValue(cls, value):
        ''' 通过value值，取消选择下拉框选项，
        @param value: 下拉框  value属性 值
        '''
        try:
            Select(cls._element()).deselect_by_value(value)
        except:
            return False
    
    @classmethod
    def MouseOver(cls):
        ''' 鼠标悬浮 '''      
        element = cls._element()                
        action = ActionChains(Web.driver)
        action.move_to_element(element)
        action.perform()
        time.sleep(1)    
    
    @classmethod
    def Click(cls):
        ''' 左键 点击 1次   '''
        
        element= cls._element()        
        action = ActionChains(Web.driver)
        action.click(element)
        action.perform()
        
    @classmethod
    def DoubleClick(cls):
        ''' 左键点击2次 '''
        
        element = cls._element()        
        action = ActionChains(Web.driver)
        action.double_click(element)
        action.perform()
                    
    @classmethod
    def EnhancedClick(cls):
        '''
        Description:
            Sometimes, one click on the element doesn't work. So wait more time, then click again and again.
        Risk:
            It may operate more than one click operations.
        '''
        
        element = cls._element()
        for _ in range(3):
            action = ActionChains(Web.driver)
            action.move_to_element(element)
            action.perform()           
            time.sleep(0.5)
    
    @classmethod
    def RightClick(cls):
        ''' 右键点击1次 '''
        
        element = cls._element()        
        action = ActionChains(Web.driver)
        action.context_click(element)
        action.perform()
       
    
    @classmethod
    def ClickAndHold(cls):
        ''' 相当于 按压，press '''
        
        element = cls._element()        
        action = ActionChains(Web.driver)
        action.click_and_hold(element)
        action.perform()
    
    
    @classmethod
    def ReleaseClick(cls):
        ''' 释放按压操作   '''
        
        element = cls._element()        
        action = ActionChains(Web.driver)
        action.release(element)
        action.perform()
        
    
    @classmethod
    def MoveAndDropTo(cls):
        '''
        @todo: move an elm to an elm where drop it to 
        '''
        raise Exception('to do')
    
    @classmethod
    def Enter(cls):
        '''     在指定输入框发送回回车键
        @note: key event -> enter
        '''
        
        element = cls._element()        
        action = ActionChains(Web.driver)
        action.send_keys_to_element(element, Keys.ENTER)
        action.perform()
            
    @classmethod
    def Ctrl(cls, key):
        """     在指定元素上执行ctrl组合键事件
        @note: key event -> control + key
        @param key: 如'X'
        """
        element = cls._element()        
        element.send_keys(Keys.CONTROL, key)
        
        
    @classmethod
    def Alt(cls, key):
        """    在指定元素上执行alt组合事件
        @note: key event ->  alt + key
        @param key: 如'X'
        """
        element = cls._element()
        element.send_keys(Keys.ALT, key)
        
    
    @classmethod
    def Space(cls):
        """     在指定输入框发送空格
        @note: key event ->  space
        """
        element = cls._element()
        element.send_keys(Keys.SPACE)
        
    
    @classmethod
    def Backspace(cls):
        """        在指定输入框发送回退键
        @note: key event ->  backspace
        """
        element = cls._element()
        element.send_keys(Keys.BACK_SPACE)
        
    
    @classmethod
    def Tab(cls):
        """        在指定输入框发送回制表键
        @note: key event ->  tab
        """
        element = cls._element()
        element.send_keys(Keys.TAB)
        
    
    @classmethod
    def Escape(cls):
        """        在指定输入框发送回退出键
        @note: key event ->  esc
        """
        element = cls._element()
        element.send_keys(Keys.ESCAPE)
            
    
    @classmethod
    def Focus(cls):
        """        在指定输入框发送 Null， 用于设置焦点
        @note: key event ->  NULL
        """
        
        element = cls._element()        
#         element.send_keys(Keys.NULL)        
        action = ActionChains(Web.driver)
        action.send_keys_to_element(element, Keys.NULL)
        action.perform()
        
    
    @classmethod
    def Upload(cls, filename):
        """        文件上传， 非原生input
        @todo:  some  upload.exe not  prepared
        @param file: 文件名(文件必须存在在工程resource目录下), upload.exe工具放在工程tools目录下
        """
        raise Exception("to do")
    
        TOOLS_PATH = ""
        RESOURCE_PATH = ""
        tool_4path = os.path.join(TOOLS_PATH, "upload.exe")        
        file_4path = os.path.join(RESOURCE_PATH, filename)
        #file_4path.decode('utf-8').encode('gbk')
        
        if os.path.isfile(file_4path):
            cls.Click()
            os.system(tool_4path + ' ' + file_4path)
        else:
            raise Exception('%s is not exists' % file_4path)
    
    @classmethod
    def UploadType(cls, file_path):
        """    上传，  一般，上传页面如果是input,原生file文件框, 如： <input type="file" id="test-image-file" name="test" accept="image/gif">，像这样的，定位到该元素，然后使用 send_keys 上传的文件的绝对路径        
        @param file_name: 文件名(文件必须存在在工程resource目录下)
        """
        if not os.path.isabs(file_path):
            return False
        
        if os.path.isfile(file_path):
            cls.SendKeys(file_path)
        else:            
            return False
        

      
    
    
    
    


#! python3
# -*- encoding: utf-8 -*-

__title__           = "rtsf-web"
__short_desc__      = "only for web ui test, base on rtsf"
__version__         = "1.2.4"
__autor__           = u"罗科峰"
__author_email__    = "lkf20031988@163.com"
__license__         = 'MIT'

HOME_PAGE           = "https://github.com/RockFeng0/rtsf-web"


# -*- encoding: utf-8 -*-
'''
Current module: rock4.softtest.web.SeleniumJar

Rough version history:
v1.0    Original version to use

********************************************************************
    @AUTHOR:  Administrator-Bruce Luo(罗科峰)
    MAIL:    lkf20031988@163.com
    RCS:      rock4.softtest.web.SeleniumJar,v 1.0 2017年2月27日
    FROM:   2017年2月27日
********************************************************************

======================================================================

UI and Web Http automation frame for python.

'''

import requests
import time,subprocess

class SeleniumJar(object):    
    def __init__(self, server_jar_full_path, java_exe_full_path = "java"):
        self._conf = {
            "java_path": java_exe_full_path,
            "jar_path": server_jar_full_path,
            }
                        
    def hub(self, port):
        ''' java -jar selenium-server.jar -role hub -port 4444
        @param port:  listen port of selenium hub 
        '''
        self._ip = "localhost"
        self._port = port 
        self.command = [self._conf["java_path"], "-jar", self._conf["jar_path"], "-port", str(port), "-role", "hub"]        
        return self
        
    def node(self,port, hub_address=("localhost", 4444)):
        ''' java -jar selenium-server.jar -role node -port 5555 -hub http://127.0.0.1:4444/grid/register/
        @param port:  listen port of selenium node
        @param hub_address: hub address which node will connect to 
        '''
        self._ip, self._port = hub_address
        self.command = [self._conf["java_path"], "-jar", self._conf["jar_path"], "-port", str(port), "-role", "node", "-hub", "http://%s:%s/grid/register/" %(self._ip, self._port)]        
        return self
    
    def start_server(self):
        """start the selenium Remote Server."""        
        self.__subp = subprocess.Popen(self.command)        
        #print("\tselenium jar pid[%s] is running." %self.__subp.pid)        
        time.sleep(2)
        
    def stop_server(self):
        """stop the selenium Remote Server
        :return:
        """
        self.__subp.kill()
        #print("\tselenium jar pid[%s] is stopped." %self.__subp.pid)
        
    def re_start_server(self):
        """reStart the selenium Remote server"""
        self.stop_server()
        self.start_server()
    
    def is_runnnig(self):
        """Determine whether hub server is running
        :return:True or False
        """
        resp = None
        try:
            resp = requests.get("http://%s:%s" %(self._ip, self._port))

            if resp.status_code == 200:
                return True
            else:
                return False
        except:
            return False
    


#! python3
# -*- encoding: utf-8 -*-
'''
Current module: setup

Rough version history:
v1.0    Original version to use

********************************************************************
    @AUTHOR:  Administrator-Bruce Luo(罗科峰)
    MAIL:    lkf20031988@163.com
    RCS:      setup,v 1.0 2018年8月19日
    FROM:   2018年8月19日
********************************************************************

======================================================================

UI and Web Http automation frame for python.

'''



from webuidriver import __about__
import os, io, sys
from shutil import rmtree
from setuptools import setup, find_packages, Command

here = os.path.abspath(os.path.dirname(__file__))

with io.open("README.md", encoding='utf-8') as f:
    long_description = f.read()

class UploadCommand(Command):
    """ Build and publish this package.
        Support setup.py upload. Copied from requests_html.
    """

    user_options = []

    @staticmethod
    def status(s):
        """Prints things in green color."""
        print("\033[0;32m{0}\033[0m".format(s))

    def initialize_options(self):
        """ override
        """
        pass

    def finalize_options(self):
        """ override
        """
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        self.status('Publishing git tags…')
        os.system('git tag v{0}'.format(__about__.__version__))
        os.system('git push --tags')

        sys.exit()

install_requires = [
    "selenium",
    "requests",
    "rtsf",
]

# dependency_links=[
# "git+https://github.com/RockFeng0/rtsf.git#egg=rtsf-0"
# ]

setup(
        name = __about__.__title__,
        version=__about__.__version__,        
        description=__about__.__short_desc__,
        long_description=long_description,
        long_description_content_type='text/markdown',
        author=__about__.__autor__,
        author_email=__about__.__author_email__,
        url=__about__.HOME_PAGE,
        license=__about__.__license__,
        python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, <4',
        packages=find_packages(exclude=()),        
        keywords='test web ui',
        install_requires=install_requires,
#         dependency_links=dependency_links,
        extras_require={},
        entry_points={
            'console_scripts': [
                'wldriver=webuidriver.cli:local_main_run', # local driver
                'wrdriver=webuidriver.cli:remote_main_run', # remote driver
                'wrhub=webuidriver.cli:hub_main_run', # selenium grid hub
                'wrnode=webuidriver.cli:node_main_run', # selenium grid node
            ]
        },
        # $ setup.py upload support.
        cmdclass={
            'upload': UploadCommand
        }
    )
